import logging
import os

import cv2
from mediapipe.python.solutions.hands import Hands

from .utils import get_letter, get_relative_path


def create_directory(path: str):
    """Create a directory if it doesn't exist."""
    if not os.path.exists(path):
        os.makedirs(path)


def initialize_capture_device(device_id: int):
    """Initialize and return the capture device."""
    return cv2.VideoCapture(device_id)


def display_instructions(frame, class_id: int):
    """Display instructions on the frame."""
    cv2.putText(
        frame,
        f"Press 'q' to collect data for class {class_id}: {get_letter(class_id)}",
        (100, 50),
        cv2.FONT_HERSHEY_SIMPLEX,
        0.6,
        (0, 255, 0),
        3,
        cv2.LINE_AA,
    )


def capture_image(cap, hands, output_dir: str):
    """Capture and save a single image if hand landmarks are detected."""
    ret, frame = cap.read()
    if not ret:
        return False

    frame_rgb = cv2.cvtColor(frame, cv2.COLOR_BGR2RGB)
    results = hands.process(frame_rgb)
    if results.multi_hand_landmarks:
        file_name = f"{len(os.listdir(output_dir)) + 1}.jpg"
        cv2.imwrite(os.path.join(output_dir, file_name), frame)
        return True
    return False


def show_image_from_file(image_path: str = ""):
    """Display an image from a file in a separate window."""
    if not image_path:
        image_path = get_relative_path("assets", "alphabet.png")
    if not os.path.exists(image_path):
        logging.warning(f"Image file {image_path} not found")
        return
    image = cv2.imread(image_path)
    if image is not None:
        cv2.imshow("Image from File", image)
        cv2.waitKey(1)
    else:
        logging.info(f"Failed to load image from {image_path}")


def collect_class_data(
    class_id: int, dataset_size: int, cap, hands: Hands, data_dir: str
):
    """Collect data for a specific class."""
    output_dir = os.path.join(data_dir, str(class_id))
    create_directory(output_dir)

    while len(os.listdir(output_dir)) < dataset_size:
        if capture_image(cap, hands, output_dir):
            cv2.imshow("frame", cap.read()[1])
            cv2.waitKey(25)


def collect_data(
    data_dir: str,
    number_of_classes: int = 26,
    dataset_size: int = 100,
    capture_device: int = 0,
):
    """Main function to collect data for all classes."""
    create_directory(data_dir)
    hands = Hands(static_image_mode=True, min_detection_confidence=0.3)
    cap = initialize_capture_device(capture_device)
    show_image_from_file()

    for class_id in range(number_of_classes):
        logging.info(f"Collecting data for class {class_id}")
        while True:
            ret, frame = cap.read()
            if not ret:
                break
            display_instructions(frame, class_id)
            cv2.imshow("frame", frame)
            if cv2.getWindowProperty("frame", cv2.WND_PROP_VISIBLE) < 1:
                cap.release()
                cv2.destroyAllWindows()
                return
            if cv2.waitKey(25) == ord("q"):
                break

        collect_class_data(class_id, dataset_size, cap, hands, data_dir)
        if cv2.getWindowProperty("frame", cv2.WND_PROP_VISIBLE) < 1:
            break

    cap.release()
    cv2.destroyAllWindows()



import logging
import os
import pickle

import cv2
import numpy as np
from mediapipe.python.solutions.hands import Hands

from .statistics import draw_class_bar_plot, print_class_stats

DATA_DIR = "./data"


def main():
    hands = Hands(max_num_hands=1, static_image_mode=True, min_detection_confidence=0.3)

    data = []
    labels = []

    for dir_ in sorted(os.listdir(DATA_DIR), key=lambda x: int(x)):
        for img_path in os.listdir(os.path.join(DATA_DIR, dir_)):
            data_aux = []
            x_, y_ = [], []
            img = cv2.imread(os.path.join(DATA_DIR, dir_, img_path))
            img_rgb = cv2.cvtColor(img, cv2.COLOR_BGR2RGB)
            results = hands.process(img_rgb)
            if results.multi_hand_landmarks:
                for hand_landmarks in results.multi_hand_landmarks:
                    for i in range(len(hand_landmarks.landmark)):
                        x_.append(hand_landmarks.landmark[i].x)
                        y_.append(hand_landmarks.landmark[i].y)

                    for i in range(len(hand_landmarks.landmark)):
                        data_aux.append(hand_landmarks.landmark[i].x - min(x_))
                        data_aux.append(hand_landmarks.landmark[i].y - min(y_))
                data.append(data_aux)
                labels.append(dir_)
            else:
                logging.info(
                    "warning hand landmakrs unrecognized in image {}/{}".format(
                        dir_, img_path
                    )
                )
    unique_labels, counts = np.unique(labels, return_counts=True)
    print_class_stats(unique_labels, counts)
    draw_class_bar_plot(unique_labels, counts)

    with open("data.pickle", "wb") as f:
        pickle.dump({"data": data, "labels": labels}, f)


main()



import logging
import os
import shutil
from typing import Optional

import typer

from .collect import collect_data
from .storage import download_all_files, download_file, list_files, upload_file
from .utils import initialize_service

logging.basicConfig(
    level=logging.DEBUG,
    format="%(asctime)s - %(name)s - %(levelname)s - %(message)s",
    handlers=[logging.StreamHandler()],
)

logger = logging.getLogger("htt")

app = typer.Typer()

DEFAULT_KEY = None


@app.command()
def collect(
    zip: bool = typer.Option(True, "--zip/--no-zip", help="Get output as zip"),
    data_dir: str = typer.Option(
        "/tmp/data", "--data-dir", "-d", help="Directory to store images"
    ),
    classes_number: int = typer.Option(
        26, "--classes-number", "-cn", help="Number of classes"
    ),
    samples_number: int = typer.Option(
        100, "--samples-number", "-sn", help="Number of images per class"
    ),
    capture_device: int = typer.Option(0, "--capture-device", help="Camera device ID"),
):
    """Take hand images."""
    collect_data(data_dir, classes_number, samples_number, capture_device)
    if zip:
        shutil.make_archive("output", "zip", data_dir)
        shutil.rmtree(data_dir)


@app.command()
def process(
    data_dir: str = typer.Option(
        "./data", "--data-dir", "-d", help="Directory with images"
    ),
):
    """Transform images into dataset pickle."""
    logging.info("not implemented %s", data_dir)


@app.command()
def upload(
    file: str = typer.Argument(
        "output.zip",
        help="Path to zip file to upload to Google Drive",
    ),
    key: Optional[str] = typer.Option(
        DEFAULT_KEY, "--key", help="Encryption key for the service account"
    ),
):
    """Upload files to Google Storage."""
    if os.path.exists(file):
        service = initialize_service(key)
        upload_file(file, service)
    else:
        logging.info(f"File {file} does not exist")


@app.command()
def list(
    key: Optional[str] = typer.Option(
        DEFAULT_KEY, "--key", help="Encryption key for the service account"
    ),
):
    """List files in Google Storage."""
    service = initialize_service(key)
    list_files(service)


@app.command()
def download(
    file_id: Optional[str] = typer.Option(
        None, "--file-id", help="File ID to download"
    ),
    key: Optional[str] = typer.Option(
        DEFAULT_KEY, "--key", help="Encryption key for the service account"
    ),
):
    """Download files from Google Drive."""
    service = initialize_service(key)
    if file_id:
        download_file(file_id, f"./{file_id}.zip", service)
    else:
        download_all_files("./dataset.zip", service)


if __name__ == "__main__":
    app()
